#!/usr/bin/python

#
# otr_factor_training	rescales flags in trainingdata by multipliing
#                       length with a given factor
#
# Author:   Christian Brandt  2013
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 

import sys
import os.path
from gamera.core import *
init_gamera()
from gamera import knn

class Config:
	infile = None
	outfile = None
	factor = 1
	def usage(self):
		usage_msg = "Usage:\n\t%s " % os.path.basename(sys.argv[0]) + \
			" -i <in.xml> -o <out.xml> -f <factor>\n" + \
                        "Multiplies all flag lengths in training data with <factor>.\n"
		sys.stderr.write(usage_msg)
		sys.exit(1)


i = 1
opt = Config()
while i < len(sys.argv):
	if sys.argv[i] == "-i":
		i += 1
		opt.infile = sys.argv[i]
	elif sys.argv[i] == "-o":
		i += 1
		opt.outfile = sys.argv[i]
	elif sys.argv[i] == "-f":
		i += 1
		opt.factor = float(sys.argv[i])
	else:
		print "unknown option"
	i += 1

if opt.infile == None:
	opt.usage()



c = knn.kNNInteractive()
d = knn.kNNInteractive()
c.change_feature_set(["nrows_feature"])
d.change_feature_set(["nrows_feature"])
c.from_xml_filename(opt.infile)

for g in c.get_glyphs():
	if g.match_id_name("*flag.*"):
		name = g.get_main_id().split(".")
		i = 1
		while i < len(name):
			name[i] = "%d" % (int(name[i]) * opt.factor)
			i += 2 # only every second number is length

		print name
		d.classify_glyph_manual(g, '.'.join(name))
		pass
	else:
		d.classify_glyph_manual(g, g.get_main_id())

d.to_xml_filename(opt.outfile)

