#!/usr/bin/python

#
# otr_german_recognize - recognizes a German tablature image
#                        and creates abc output
#                        this file is part of the OTR toolkit for gamera
#
# Authors:  Christine Pranzas 2007
#           Christoph Dalitz  2008
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 

from gamera.core import *
from gamera.util import *
import sys
import os
import re
import random
from gamera.plugins.image_utilities import union_images
from os.path import *
from math import sqrt,log
from gamera.toolkits.otr import *
from gamera.toolkits.otr import *
import time

from gamera.classify import BasicGroupingFunction, ShapedGroupingFunction, BoundingBoxGroupingFunction


class c_opt:
    trainfile=""
    imagefile = []
    outfile = ""
    onlines = 0
    debug=0 
    nlines = 0
    num_k = 1
    smooth = 0
    corr_rot = 0
    remove_border = False
    specklesize=7
    weightfile=""
    clean=False
    log_file=None
    dot_to_flag=True
    bar_det=True
    abcmusic=False
    title=""
    tuning = "gdAFCG,"
		
    def error_exit(self):
        usage = "Usage:\n\t" + sys.argv[0] + " [<options>] <infile>\n" + \
                "Options:\n" + \
                "\t-o        output file name ([basename + '.abc'])\n" + \
                "\t-d <xml>  training data file ['" + self.trainfile + "']\n" + \
                "\t-w <xml>  feature weightfile ['" + self.weightfile + "']\n" + \
                "\t-k <k>    value of k in kNN ([" + ("%d" % opt.num_k) +"])\n" + \
                "\t-debug <n> debug level (o,1 or 2) ['" + ("%d" % self.debug) + "']\n" + \
                "\t-c        'clean lines'. This option will extrapolate the found\n" + \
                "\t          separator lines and remove everything around them.\n" + \
                "\t-speckle <n> remove CCs up to size <n> ['" + ("%d" % self.specklesize) + "']\n" + \
                "\t          to suppress despeckling, set <n> to 0\n" + \
                "\t-deskew   correct rotation\n" + \
                "\t-border   remove copy border\n" + \
                "\t-knnbars  detect bars with training data rather than heuristically\n" + \
                "\t-t <text> song title in abc output ([inputfilename])\n" + \
                "\t-l <log>  write log messages to file <log>\n" + \
                "\t-abcmusic write music transcription in addition to tabcode\n" + \
                "\t-tuning <tuning>\n" + \
                "\t          tuning for music transcription, given from top\n" + \
                "\t          to bottom. Default is \"" + self.tuning + "\"\n"
                
        sys.stderr.write(usage)
        sys.exit(1)
opt = c_opt()

i = 1
while i < len(sys.argv):
    if sys.argv[i] == "-o":
        i += 1; opt.outfile = sys.argv[i]
    elif sys.argv[i] == "-d":
        i += 1; opt.trainfile = sys.argv[i]
    elif sys.argv[i] == "-w":
        i += 1; opt.weightfile = sys.argv[i]
    elif sys.argv[i] == "-k":
        i += 1; opt.num_k = int(sys.argv[i])
    elif sys.argv[i] == "-t":
        i += 1; opt.title = sys.argv[i]
    elif sys.argv[i] == "-speckle":
        i += 1; opt.speckle_size = int(sys.argv[i])
    elif sys.argv[i] == '-deskew':
        opt.corr_rot = 1
    elif sys.argv[i] == '-border':
        opt.remove_border = True
    elif sys.argv[i] == '-debug':
        i+=1; opt.debug = int(sys.argv[i])
    elif sys.argv[i] == '-c':
        opt.clean = True
    elif sys.argv[i] == '-l':
        i += 1; opt.log_file = sys.argv[i]
    elif sys.argv[i] == '-abcmusic':
        opt.abcmusic = True
    elif sys.argv[i] == "-tuning":
        i += 1; opt.tuning = sys.argv[i]
    elif sys.argv[i] == '-knnbars':
        opt.bar_det = False
    elif sys.argv[i][0] == '-':
        opt.error_exit()
    else:
        if exists(sys.argv[i]) and "findlines.png" not in sys.argv[i] and "despeckle.png" not in sys.argv[i] and "groundtruth.png" not in sys.argv[i] :
            if isfile(sys.argv[i]) and (sys.argv[i].lower().endswith(".png") or  sys.argv[i].lower().endswith(".tiff") or sys.argv[i].lower().endswith(".tif") ):
                opt.imagefile.append(sys.argv[i])
            elif isdir(sys.argv[i]):
                if sys.argv[i][-1]!=os.sep:
                    sys.argv[i]+=os.sep
                    for filen in os.listdir(sys.argv[i]):
                        filename=filen.split(".")
                        if (filename[-1]=="png" or filename[-1]=="tiff" or filename[-1]=="tif" ) and "findlines.png" not in filen and "despeckle.png" not in filen and "groundtruth.png" not in filen :
                            opt.imagefile.append(sys.argv[i]+filen)
		
						
    i += 1
     
if len(opt.imagefile)==0:
    opt.error_exit()

# prevent gamera from parsing arguments
sys.argv = []

init_gamera()


for imagefile in opt.imagefile:
    print imagefile
    image=load_image(imagefile)

    if opt.title!="" and len(opt.imagefile)==1:
        pass
    else:
        li=imagefile.split("/")
        opt.title=li[-1]

    if opt.outfile=="":
        strings =imagefile.split(".")
        strings[-1]="abc"
        opt.outfile=".".join(strings)
        
    
    if opt.trainfile=="":
        print "Error! No Trainfile!"
        break
    
    if image.data.pixel_type != ONEBIT:
        image = image.to_onebit()
    if opt.corr_rot:
        print "rotation correction...",
        sys.stdout.flush()
        image=image.deskew()
        print "done"
   
    if opt.specklesize > 0:
        print "despeckling...",
        sys.stdout.flush()
        image.despeckle(opt.specklesize)
        image.invert()
        image.despeckle(opt.specklesize)
        image.invert()
        print "done"

    if opt.smooth < 3:
        print "smoothing...",
        sys.stdout.flush()
        image=image.smooth_onebit(0,opt.smooth)
        print "done"
    if opt.remove_border:
        print "removing copy borders...",
        sys.stdout.flush()
        image=image.remove_copy_border()
        print "done"

    germanpage=GermanLutePage(image,opt.bar_det,opt.clean,opt.log_file,opt.debug)

    image=germanpage.image.to_onebit()
    rgb_image=image.to_rgb()
    
    if opt.debug >= 1:
        print "Initialize Classifier......",
    if opt.weightfile=="":    
        classifier=knn.kNNInteractive([], [\
            'aspect_ratio',\
            'moments',\
            'nrows_feature',\
            'ncols_feature',\
            'volume64regions',\
            ], 0)
        classifier.num_k = opt.num_k
        classifier.from_xml_filename(opt.trainfile)
    else:
        classifier=knn.kNNInteractive([],"all")
        classifier.num_k=opt.num_k
        classifier.from_xml_filename(opt.trainfile)
        classifier.load_settings(opt.weightfile)
    if opt.debug >= 1:
        print "done.",
    #
    # classify the connected components of the image, use half of oligon_height
    # to decide about grouped glyphs
    # Note: we must remove "lyrics" classes from the training data
    #
    ccs=[]
    ccs=image.cc_analysis()
    grp_distance = max([germanpage.line_height,5])
    #grp_distance=4
    added, removed=classifier.group_list_automatic(ccs,\
            ShapedGroupingFunction(grp_distance),max_parts_per_group=5)

    #
    # process groups detected by gamera's grouping algorithm
    # and remove group.parts and trash
    #

        
    if len(added) > 0:
        print "Added",len(added)
        ccs.extend(added)

    ccs=[x for x in ccs if \
         (not "_group._part." in x.get_main_id()) and \
         (not "_group." in x.get_main_id()) and \
         (not "_part." in x.get_main_id()) and \
         (not "trash" in x.get_main_id())]

    germanneumes=GermanGlyph(image,ccs,germanpage.straight_lines,germanpage.bars,germanpage.line_height,germanpage.distance_between_two_lines,germanpage.character_height,opt.dot_to_flag,opt.title,opt.debug)

    f=open(opt.outfile,"w")
    f.write(germanneumes.song)
   
    if opt.abcmusic:
        music=germanneumes.to_abcmusic(tuning=opt.tuning,factor=0)
        f.write("\n")
        f.write(music)

    f.close()

    if opt.debug==2:
        #write debug images
        image=germanpage.mark_lines()
        image.save_PNG("debug_mark_lines.png")
        image=germanpage.mark_bars()
        image.save_PNG("debug_mark_bars.png")
        image=germanneumes.mark_flag_groups()
        image.save_PNG("debug_mark_flag_groups.png")
        image=germanneumes.mark_note_groups()
        image.save_PNG("debug_mark_note_groups.png")


                    

    

